#include <Muon/Muon.h>
#include "m6502.h"

struct M6502TransferRegistgerTests
{
public:
	m6502::Mem mem;
	m6502::CPU cpu;

	virtual void SetUp()
	{
		cpu.Reset( mem );
	}

	virtual void TearDown()
	{
	}

	void ExpectUnaffectedRegisters( m6502::CPU CPUBefore )
	{
		CHECK_EQ( CPUBefore.Flag.C, cpu.Flag.C );
		CHECK_EQ( CPUBefore.Flag.I, cpu.Flag.I );
		CHECK_EQ( CPUBefore.Flag.D, cpu.Flag.D );
		CHECK_EQ( CPUBefore.Flag.B, cpu.Flag.B );
		CHECK_EQ( CPUBefore.Flag.V, cpu.Flag.V );
	}
};

TEST_F( M6502TransferRegistgerTests, TAXCanTransferANonNegativeNonZeroValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.A = 0x42;
	cpu.X = 0x32;
	cpu.Flag.Z = true;
	cpu.Flag.N = true;
	mem[0xFF00] = CPU::INS_TAX;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.A, 0x42 );
	CHECK_EQ( cpu.X, 0x42 );
	CHECK_FALSE( cpu.Flag.Z );
	CHECK_FALSE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TAXCanTransferANonNegativeZeroValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.A = 0x0;
	cpu.X = 0x32;
	cpu.Flag.Z = false;
	cpu.Flag.N = true;
	mem[0xFF00] = CPU::INS_TAX;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.A, 0x0 );
	CHECK_EQ( cpu.X, 0x0 );
	CHECK_TRUE( cpu.Flag.Z );
	CHECK_FALSE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TAXCanTransferANegativeValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.A = 0b10001011;
	cpu.X = 0x32;
	cpu.Flag.Z = true;
	cpu.Flag.N = false;
	mem[0xFF00] = CPU::INS_TAX;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.A, 0b10001011 );
	CHECK_EQ( cpu.X, 0b10001011 );
	CHECK_FALSE( cpu.Flag.Z );
	CHECK_TRUE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TAYCanTransferANonNegativeNonZeroValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.A = 0x42;
	cpu.Y = 0x32;
	cpu.Flag.Z = true;
	cpu.Flag.N = true;
	mem[0xFF00] = CPU::INS_TAY;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.A, 0x42 );
	CHECK_EQ( cpu.Y, 0x42 );
	CHECK_FALSE( cpu.Flag.Z );
	CHECK_FALSE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TAYCanTransferANonNegativeZeroValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.A = 0x0;
	cpu.Y = 0x32;
	cpu.Flag.Z = false;
	cpu.Flag.N = true;
	mem[0xFF00] = CPU::INS_TAY;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.A, 0x0 );
	CHECK_EQ( cpu.Y, 0x0 );
	CHECK_TRUE( cpu.Flag.Z );
	CHECK_FALSE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TAYCanTransferANegativeValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.A = 0b10001011;
	cpu.Y = 0x32;
	cpu.Flag.Z = true;
	cpu.Flag.N = false;
	mem[0xFF00] = CPU::INS_TAY;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.A, 0b10001011 );
	CHECK_EQ( cpu.Y, 0b10001011 );
	CHECK_FALSE( cpu.Flag.Z );
	CHECK_TRUE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TXACanTransferANonNegativeNonZeroValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.X = 0x42;
	cpu.A = 0x32;
	cpu.Flag.Z = true;
	cpu.Flag.N = true;
	mem[0xFF00] = CPU::INS_TXA;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.X, 0x42 );
	CHECK_EQ( cpu.A, 0x42 );
	CHECK_FALSE( cpu.Flag.Z );
	CHECK_FALSE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TXACanTransferANonNegativeZeroValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.X = 0x0;
	cpu.A = 0x32;
	cpu.Flag.Z = false;
	cpu.Flag.N = true;
	mem[0xFF00] = CPU::INS_TXA;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.X, 0x0 );
	CHECK_EQ( cpu.A, 0x0 );
	CHECK_TRUE( cpu.Flag.Z );
	CHECK_FALSE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TXACanTransferANegativeValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.X = 0b10001011;
	cpu.A = 0x32;
	cpu.Flag.Z = true;
	cpu.Flag.N = false;
	mem[0xFF00] = CPU::INS_TXA;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.X, 0b10001011 );
	CHECK_EQ( cpu.A, 0b10001011 );
	CHECK_FALSE( cpu.Flag.Z );
	CHECK_TRUE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TYACanTransferANonNegativeNonZeroValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.Y = 0x42;
	cpu.A = 0x32;
	cpu.Flag.Z = true;
	cpu.Flag.N = true;
	mem[0xFF00] = CPU::INS_TYA;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.Y, 0x42 );
	CHECK_EQ( cpu.A, 0x42 );
	CHECK_FALSE( cpu.Flag.Z );
	CHECK_FALSE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TYACanTransferANonNegativeZeroValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.Y = 0x0;
	cpu.A = 0x32;
	cpu.Flag.Z = false;
	cpu.Flag.N = true;
	mem[0xFF00] = CPU::INS_TYA;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.Y, 0x0 );
	CHECK_EQ( cpu.A, 0x0 );
	CHECK_TRUE( cpu.Flag.Z );
	CHECK_FALSE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}

TEST_F( M6502TransferRegistgerTests, TYACanTransferANegativeValue )
{
	// given:
	using namespace m6502;
	cpu.Reset( 0xFF00, mem );
	cpu.Y = 0b10001011;
	cpu.A = 0x32;
	cpu.Flag.Z = true;
	cpu.Flag.N = false;
	mem[0xFF00] = CPU::INS_TYA;
	constexpr s32 EXPECTED_CYCLES = 2;
	CPU CPUCopy = cpu;

	// when:
	const s32 ActualCycles = cpu.Execute( EXPECTED_CYCLES, mem );

	// then:
	CHECK_EQ( ActualCycles, EXPECTED_CYCLES );
	CHECK_EQ( cpu.Y, 0b10001011 );
	CHECK_EQ( cpu.A, 0b10001011 );
	CHECK_FALSE( cpu.Flag.Z );
	CHECK_TRUE( cpu.Flag.N );
	ExpectUnaffectedRegisters( CPUCopy );
}